@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -134,6 +134,9 @@ def pytest_terminal_summary(terminalreporter):
         terminalreporter.write_line(
             '  - Typical runtimes: %s' % (statistics.runtimes,)
         )
+        terminalreporter.write_line(
+            '  - Fraction of time spent in data generation: %s' % (
+                statistics.draw_time_percentage,))
         terminalreporter.write_line(
             '  - Stopped because %s' % (statistics.exit_reason,)
         )
