@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -19,78 +19,62 @@ from __future__ import division, print_function, absolute_import
 
 import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import OrderedDict, hbytes
+from hypothesis.internal.compat import OrderedDict
+from hypothesis.internal.conjecture.utils import combine_labels
 from hypothesis.searchstrategy.strategies import SearchStrategy, \
-    MappedSearchStrategy, one_of_strategies
+    MappedSearchStrategy
 
 
 class TupleStrategy(SearchStrategy):
-
     """A strategy responsible for fixed length tuples based on heterogenous
     strategies for each of their elements."""
 
-    def __init__(self,
-                 strategies, tuple_type):
+    def __init__(self, strategies):
         SearchStrategy.__init__(self)
-        strategies = tuple(strategies)
-        self.element_strategies = strategies
+        self.element_strategies = tuple(strategies)
 
     def do_validate(self):
         for s in self.element_strategies:
             s.validate()
 
+    def calc_label(self):
+        return combine_labels(
+            self.class_label, *[s.label for s in self.element_strategies])
+
     def __repr__(self):
         if len(self.element_strategies) == 1:
             tuple_string = '%s,' % (repr(self.element_strategies[0]),)
         else:
             tuple_string = ', '.join(map(repr, self.element_strategies))
-        return 'TupleStrategy((%s))' % (
-            tuple_string,
-        )
+        return 'TupleStrategy((%s))' % (tuple_string,)
 
     def calc_has_reusable_values(self, recur):
         return all(recur(e) for e in self.element_strategies)
 
-    def newtuple(self, xs):
-        """Produce a new tuple of the correct type."""
-        return tuple(xs)
-
     def do_draw(self, data):
-        return self.newtuple(
-            data.draw(e) for e in self.element_strategies
-        )
+        return tuple(data.draw(e) for e in self.element_strategies)
 
     def calc_is_empty(self, recur):
         return any(recur(e) for e in self.element_strategies)
 
 
-TERMINATOR = hbytes(b'\0')
-
-
 class ListStrategy(SearchStrategy):
+    """A strategy for lists which takes a strategy for its elements and the
+    allowed lengths, and generates lists with the correct size and contents."""
 
-    """A strategy for lists which takes an intended average length and a
-    strategy for each of its element types and generates lists containing any
-    of those element types.
-
-    The conditional distribution of the length is geometric, and the
-    conditional distribution of each parameter is whatever their
-    strategies define.
-
-    """
-
-    def __init__(
-        self,
-        strategies, average_length=50.0, min_size=0, max_size=float('inf')
-    ):
+    def __init__(self, elements, min_size=0, max_size=float('inf')):
         SearchStrategy.__init__(self)
-
-        assert average_length > 0
-        self.average_length = average_length
-        strategies = tuple(strategies)
         self.min_size = min_size or 0
         self.max_size = max_size or float('inf')
-        self.element_strategy = one_of_strategies(strategies)
+        assert 0 <= min_size <= max_size
+        self.average_size = min(
+            max(self.min_size * 2, self.min_size + 5),
+            0.5 * (self.min_size + self.max_size),
+        )
+        self.element_strategy = elements
+
+    def calc_label(self):
+        return combine_labels(self.class_label, self.element_strategy.label)
 
     def do_validate(self):
         self.element_strategy.validate()
@@ -99,6 +83,13 @@ class ListStrategy(SearchStrategy):
                 'Cannot create non-empty lists with elements drawn from '
                 'strategy %r because it has no values.') % (
                 self.element_strategy,))
+        if self.element_strategy.is_empty and 0 < self.max_size < float('inf'):
+            from hypothesis._settings import note_deprecation
+            note_deprecation(
+                'Cannot create a collection of max_size=%r, because no '
+                'elements can be drawn from the element strategy %r'
+                % (self.max_size, self.element_strategy)
+            )
 
     def calc_is_empty(self, recur):
         if self.min_size == 0:
@@ -114,7 +105,7 @@ class ListStrategy(SearchStrategy):
         elements = cu.many(
             data,
             min_size=self.min_size, max_size=self.max_size,
-            average_size=self.average_length
+            average_size=self.average_size
         )
         result = []
         while elements.more():
@@ -122,43 +113,18 @@ class ListStrategy(SearchStrategy):
         return result
 
     def __repr__(self):
-        return (
-            'ListStrategy(%r, min_size=%r, average_size=%r, max_size=%r)'
-        ) % (
-            self.element_strategy, self.min_size, self.average_length,
+        return '%s(%r, min_size=%r, max_size=%r)' % (
+            self.__class__.__name__, self.element_strategy, self.min_size,
             self.max_size
         )
 
 
-class UniqueListStrategy(SearchStrategy):
+class UniqueListStrategy(ListStrategy):
 
-    def __init__(
-        self,
-        elements, min_size, max_size, average_size,
-        key
-    ):
-        super(UniqueListStrategy, self).__init__()
-        assert min_size <= average_size <= max_size
-        self.min_size = min_size
-        self.max_size = max_size
-        self.average_size = average_size
-        self.element_strategy = elements
+    def __init__(self, elements, min_size, max_size, key):
+        super(UniqueListStrategy, self).__init__(elements, min_size, max_size)
         self.key = key
 
-    def do_validate(self):
-        self.element_strategy.validate()
-        if self.is_empty:
-            raise InvalidArgument((
-                'Cannot create non-empty lists with elements drawn from '
-                'strategy %r because it has no values.') % (
-                self.element_strategy,))
-
-    def calc_is_empty(self, recur):
-        if self.min_size == 0:
-            return False
-        else:
-            return recur(self.element_strategy)
-
     def do_draw(self, data):
         if self.element_strategy.is_empty:
             assert self.min_size == 0
@@ -185,13 +151,11 @@ class UniqueListStrategy(SearchStrategy):
 
 
 class FixedKeysDictStrategy(MappedSearchStrategy):
-
     """A strategy which produces dicts with a fixed set of keys, given a
     strategy for each of their equivalent values.
 
-    e.g. {'foo' : some_int_strategy} would
-    generate dicts with the single key 'foo' mapping to some integer.
-
+    e.g. {'foo' : some_int_strategy} would generate dicts with the single
+    key 'foo' mapping to some integer.
     """
 
     def __init__(self, strategy_dict):
@@ -210,7 +174,7 @@ class FixedKeysDictStrategy(MappedSearchStrategy):
                 ))
         super(FixedKeysDictStrategy, self).__init__(
             strategy=TupleStrategy(
-                (strategy_dict[k] for k in self.keys), tuple
+                (strategy_dict[k] for k in self.keys)
             )
         )
 
